@@ -1,11 +1,17 @@
 use never::Never;
-use std::{error::Error, ops::Deref};
+use std::{
+    error::Error,
+    fmt::{self, Debug},
+    ops::Deref,
+    sync::Arc,
+};
 
 pub use crate::filter::filter;
 pub use crate::flatten::flatten;
 pub use crate::for_each::for_each;
 pub use crate::from_iter::from_iter;
 pub use crate::map::map;
+pub use crate::merge::merge;
 pub use crate::scan::scan;
 pub use crate::skip::skip;
 pub use crate::take::take;
@@ -15,6 +21,8 @@ mod flatten;
 mod for_each;
 mod from_iter;
 mod map;
+mod merge;
+mod pipe;
 mod scan;
 mod skip;
 mod take;
@@ -22,11 +30,12 @@ mod take;
 /// A message passed to a [`Callbag`].
 ///
 /// See <https://github.com/callbag/callbag/blob/9020d6f68f31034a717465dce38235df749f3353/types.d.ts#L12-L22>
+#[derive(Clone, Debug)]
 pub enum Message<I, O> {
-    Handshake(Callbag<O, I>),
+    Handshake(Arc<Callbag<O, I>>),
     Data(I),
     Pull,
-    Error(Box<dyn Error + Send + Sync + 'static>),
+    Error(Arc<dyn Error + Send + Sync + 'static>),
     Terminate,
 }
 
@@ -47,6 +56,25 @@ pub type Sink<T> = Callbag<T, Never>;
 
 pub type CallbagFn<I, O> = Box<dyn Fn(Message<I, O>) + Send + Sync>;
 
+impl<I, O> Deref for Callbag<I, O> {
+    type Target = CallbagFn<I, O>;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
+impl<I, O> Debug for Callbag<I, O> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(
+            f,
+            "Callbag<{}, {}>",
+            std::any::type_name::<I>(),
+            std::any::type_name::<O>(),
+        )
+    }
+}
+
 impl<I, O, F: 'static> From<F> for Callbag<I, O>
 where
     F: Fn(Message<I, O>) + Send + Sync,
@@ -55,11 +83,3 @@ where
         Callbag(Box::new(handler))
     }
 }
-
-impl<I, O> Deref for Callbag<I, O> {
-    type Target = CallbagFn<I, O>;
-
-    fn deref(&self) -> &Self::Target {
-        &self.0
-    }
-}
